using System;
using System.ComponentModel;
using System.Drawing;
using System.Runtime.InteropServices;
using System.Threading;
using System.Windows.Forms;
using Microsoft.Win32;

namespace UI
{
	public class Main : Form
	{
		public abstract class MappedDouble : INotifyPropertyChanged
		{
			private double _value;

			public abstract int IntValue { get; set; }

			public abstract string StrValue { get; set; }

			public double Value
			{
				get
				{
					return _value;
				}
				set
				{
					_value = value;
					this.PropertyChanged(this, new PropertyChangedEventArgs("Value"));
				}
			}

			public event PropertyChangedEventHandler PropertyChanged = delegate
			{
			};
		}

		public class ResValue : MappedDouble
		{
			private const int MinValue = 128;

			public override int IntValue
			{
				get
				{
					return (int)Math.Round(Math.Log(base.Value / 128.0) / Math.Log(2.0));
				}
				set
				{
					base.Value = 128 << value;
				}
			}

			public override string StrValue
			{
				get
				{
					return base.Value.ToString();
				}
				set
				{
					base.Value = double.Parse(value);
				}
			}

			public ResValue()
			{
				base.Value = 128.0;
			}
		}

		public class NrmValue : MappedDouble
		{
			private double _min;

			private double _max;

			private string _strBackup;

			public override int IntValue
			{
				get
				{
					return Math.Max(Math.Min((int)Math.Round(10.0 * (base.Value - _min) / (_max - _min)), 10), 0);
				}
				set
				{
					base.Value = (double)value / 10.0 * (_max - _min) + _min;
					_strBackup = base.Value.ToString();
				}
			}

			public override string StrValue
			{
				get
				{
					return _strBackup;
				}
				set
				{
					_strBackup = value;
					try
					{
						base.Value = double.Parse(value);
					}
					catch (Exception)
					{
					}
				}
			}

			public NrmValue(double min, double max)
			{
				base.Value = 0.0;
				_min = min;
				_max = max;
			}
		}

		private ResValue _resolutionValue = new ResValue();

		private NrmValue _contrastValue = new NrmValue(0.0, 1.0);

		private NrmValue _thresholdValue = new NrmValue(0.0, 1.0);

		private IContainer components;

		private Label label1;

		private Label label2;

		private Label label3;

		private TextBox _gmtInput;

		private Button _pickInput;

		private TrackBar _threshSlider;

		private TrackBar _contrastSlider;

		private Button _export;

		private Button _pickOutput;

		private TextBox _pngOutput;

		private Label label4;

		private TrackBar _resSlider;

		private Label label5;

		private TextBox _threshText;

		private TextBox _contrastText;

		private TextBox _resText;

		public event PropertyChangedEventHandler PropertyChanged = delegate
		{
		};

		[DllImport("Baker.dll")]
		private static extern void BakeGMT(string input, string output, float threshold, float contrast, int size);

		public Main()
		{
			InitializeComponent();
			_threshText.DataBindings.Add("Text", _thresholdValue, "StrValue", formattingEnabled: false, DataSourceUpdateMode.OnPropertyChanged);
			_threshSlider.DataBindings.Add("Value", _thresholdValue, "IntValue", formattingEnabled: false, DataSourceUpdateMode.OnPropertyChanged);
			_resText.DataBindings.Add("Text", _resolutionValue, "StrValue", formattingEnabled: false, DataSourceUpdateMode.OnPropertyChanged);
			_resSlider.DataBindings.Add("Value", _resolutionValue, "IntValue", formattingEnabled: false, DataSourceUpdateMode.OnPropertyChanged);
			_contrastText.DataBindings.Add("Text", _contrastValue, "StrValue", formattingEnabled: false, DataSourceUpdateMode.OnPropertyChanged);
			_contrastSlider.DataBindings.Add("Value", _contrastValue, "IntValue", formattingEnabled: false, DataSourceUpdateMode.OnPropertyChanged);
			ReadSettings(Registry.CurrentUser.CreateSubKey("Software\\Image Space Incorporated\\Baker"));
		}

		private void Export(object sender, EventArgs e)
		{
            try
            {
				BakeGMT(_gmtInput.Text, _pngOutput.Text, (float)_thresholdValue.Value, (float)_contrastValue.Value, (int)_resolutionValue.Value);

			}
            catch (Exception er)
            {
				MessageBox.Show(er.Message,"Error in Baker, returning to UI");
            }
			
			WriteSettings();
		}

		private void RunBaker()
        {


        }

		private void ReadSettings(RegistryKey key)
		{
			_contrastValue.StrValue = (string)key.GetValue("EdgeMap.Contrast", "0.5");
			_thresholdValue.StrValue = (string)key.GetValue("EdgeMap.Threshold", "0.0");
			_resolutionValue.StrValue = (string)key.GetValue("EdgeMap.Resolution", "1024.0");
			_gmtInput.Text = (string)key.GetValue("InputPath", "");
			_pngOutput.Text = (string)key.GetValue("OutputPath", "");
		}

		private void WriteSettings()
		{
			Registry.SetValue("HKEY_CURRENT_USER\\Software\\Image Space Incorporated\\Baker\\", "EdgeMap.Contrast", _contrastValue.Value);
			Registry.SetValue("HKEY_CURRENT_USER\\Software\\Image Space Incorporated\\Baker\\", "EdgeMap.Threshold", _thresholdValue.Value);
			Registry.SetValue("HKEY_CURRENT_USER\\Software\\Image Space Incorporated\\Baker\\", "EdgeMap.Resolution", _resolutionValue.Value);
			Registry.SetValue("HKEY_CURRENT_USER\\Software\\Image Space Incorporated\\Baker\\", "InputPath", _gmtInput.Text);
			Registry.SetValue("HKEY_CURRENT_USER\\Software\\Image Space Incorporated\\Baker\\", "OutputPath", _pngOutput.Text);
		}

		private void PickInput(object sender, EventArgs e)
		{
			OpenFileDialog openFileDialog = new OpenFileDialog();
			openFileDialog.Filter = "GMT Files|*.gmt|Any File|*.*";
			if (DialogResult.OK == openFileDialog.ShowDialog())
			{
				_gmtInput.Text = openFileDialog.FileName;
			}
		}

		private void PickOutput(object sender, EventArgs e)
		{
			SaveFileDialog saveFileDialog = new SaveFileDialog();
			saveFileDialog.Filter = "PNG Files|*.png|Any File|*.*";
			if (DialogResult.OK == saveFileDialog.ShowDialog())
			{
				_pngOutput.Text = saveFileDialog.FileName;
			}
		}

		protected override void Dispose(bool disposing)
		{
			if (disposing && components != null)
			{
				components.Dispose();
			}
			base.Dispose(disposing);
		}

		private void InitializeComponent()
		{
			label1 = new System.Windows.Forms.Label();
			label2 = new System.Windows.Forms.Label();
			label3 = new System.Windows.Forms.Label();
			_gmtInput = new System.Windows.Forms.TextBox();
			_pickInput = new System.Windows.Forms.Button();
			_threshSlider = new System.Windows.Forms.TrackBar();
			_contrastSlider = new System.Windows.Forms.TrackBar();
			_export = new System.Windows.Forms.Button();
			_pickOutput = new System.Windows.Forms.Button();
			_pngOutput = new System.Windows.Forms.TextBox();
			label4 = new System.Windows.Forms.Label();
			_resSlider = new System.Windows.Forms.TrackBar();
			label5 = new System.Windows.Forms.Label();
			_threshText = new System.Windows.Forms.TextBox();
			_contrastText = new System.Windows.Forms.TextBox();
			_resText = new System.Windows.Forms.TextBox();
			((System.ComponentModel.ISupportInitialize)_threshSlider).BeginInit();
			((System.ComponentModel.ISupportInitialize)_contrastSlider).BeginInit();
			((System.ComponentModel.ISupportInitialize)_resSlider).BeginInit();
			SuspendLayout();
			label1.AutoSize = true;
			label1.Location = new System.Drawing.Point(14, 16);
			label1.Name = "label1";
			label1.Size = new System.Drawing.Size(53, 13);
			label1.TabIndex = 0;
			label1.Text = "GMT File:";
			label2.AutoSize = true;
			label2.Location = new System.Drawing.Point(10, 88);
			label2.Name = "label2";
			label2.Size = new System.Drawing.Size(57, 13);
			label2.TabIndex = 1;
			label2.Text = "Threshold:";
			label3.AutoSize = true;
			label3.Location = new System.Drawing.Point(18, 135);
			label3.Name = "label3";
			label3.Size = new System.Drawing.Size(49, 13);
			label3.TabIndex = 2;
			label3.Text = "Contrast:";
			_gmtInput.Location = new System.Drawing.Point(73, 13);
			_gmtInput.Name = "_gmtInput";
			_gmtInput.Size = new System.Drawing.Size(435, 20);
			_gmtInput.TabIndex = 3;
			_pickInput.Location = new System.Drawing.Point(514, 11);
			_pickInput.Name = "_pickInput";
			_pickInput.Size = new System.Drawing.Size(75, 23);
			_pickInput.TabIndex = 6;
			_pickInput.Text = "...";
			_pickInput.UseVisualStyleBackColor = true;
			_pickInput.Click += new System.EventHandler(PickInput);
			_threshSlider.Location = new System.Drawing.Point(73, 81);
			_threshSlider.Name = "_threshSlider";
			_threshSlider.Size = new System.Drawing.Size(435, 45);
			_threshSlider.TabIndex = 7;
			_contrastSlider.Location = new System.Drawing.Point(73, 132);
			_contrastSlider.Name = "_contrastSlider";
			_contrastSlider.Size = new System.Drawing.Size(435, 45);
			_contrastSlider.TabIndex = 8;
			_export.Location = new System.Drawing.Point(10, 237);
			_export.Name = "_export";
			_export.Size = new System.Drawing.Size(580, 23);
			_export.TabIndex = 9;
			_export.Text = "Export";
			_export.UseVisualStyleBackColor = true;
			_export.Click += new System.EventHandler(Export);
			_pickOutput.Location = new System.Drawing.Point(514, 37);
			_pickOutput.Name = "_pickOutput";
			_pickOutput.Size = new System.Drawing.Size(75, 23);
			_pickOutput.TabIndex = 12;
			_pickOutput.Text = "...";
			_pickOutput.UseVisualStyleBackColor = true;
			_pickOutput.Click += new System.EventHandler(PickOutput);
			_pngOutput.Location = new System.Drawing.Point(73, 39);
			_pngOutput.Name = "_pngOutput";
			_pngOutput.Size = new System.Drawing.Size(435, 20);
			_pngOutput.TabIndex = 11;
			label4.AutoSize = true;
			label4.Location = new System.Drawing.Point(25, 42);
			label4.Name = "label4";
			label4.Size = new System.Drawing.Size(42, 13);
			label4.TabIndex = 10;
			label4.Text = "Output:";
			_resSlider.LargeChange = 3;
			_resSlider.Location = new System.Drawing.Point(73, 183);
			_resSlider.Maximum = 6;
			_resSlider.Name = "_resSlider";
			_resSlider.Size = new System.Drawing.Size(435, 45);
			_resSlider.TabIndex = 15;
			label5.AutoSize = true;
			label5.Location = new System.Drawing.Point(7, 186);
			label5.Name = "label5";
			label5.Size = new System.Drawing.Size(60, 13);
			label5.TabIndex = 13;
			label5.Text = "Resolution:";
			_threshText.Location = new System.Drawing.Point(514, 81);
			_threshText.Name = "_threshText";
			_threshText.Size = new System.Drawing.Size(65, 20);
			_threshText.TabIndex = 4;
			_contrastText.Location = new System.Drawing.Point(514, 132);
			_contrastText.Name = "_contrastText";
			_contrastText.Size = new System.Drawing.Size(65, 20);
			_contrastText.TabIndex = 5;
			_resText.Location = new System.Drawing.Point(514, 183);
			_resText.Name = "_resText";
			_resText.Size = new System.Drawing.Size(65, 20);
			_resText.TabIndex = 14;
			base.AutoScaleDimensions = new System.Drawing.SizeF(6f, 13f);
			base.AutoScaleMode = System.Windows.Forms.AutoScaleMode.Font;
			base.ClientSize = new System.Drawing.Size(602, 272);
			base.Controls.Add(_resSlider);
			base.Controls.Add(_resText);
			base.Controls.Add(label5);
			base.Controls.Add(_pickOutput);
			base.Controls.Add(_pngOutput);
			base.Controls.Add(label4);
			base.Controls.Add(_export);
			base.Controls.Add(_contrastSlider);
			base.Controls.Add(_threshSlider);
			base.Controls.Add(_pickInput);
			base.Controls.Add(_contrastText);
			base.Controls.Add(_threshText);
			base.Controls.Add(_gmtInput);
			base.Controls.Add(label3);
			base.Controls.Add(label2);
			base.Controls.Add(label1);
			base.Name = "Main";
			Text = "Edge Baker";
			((System.ComponentModel.ISupportInitialize)_threshSlider).EndInit();
			((System.ComponentModel.ISupportInitialize)_contrastSlider).EndInit();
			((System.ComponentModel.ISupportInitialize)_resSlider).EndInit();
			ResumeLayout(false);
			PerformLayout();
		}
	}
}
